@@ -229,7 +229,7 @@ dodgr_vertices <- function (graph) {
         if (is.na (gr_cols$edge_id)) {
             hash <- "" # nocov
         } else {
-            hash <- digest::digest (graph [[gr_cols$edge_id]])
+            hash <- get_hash (graph, contracted = FALSE, force = TRUE)
         }
     }
 
@@ -294,7 +294,7 @@ dodgr_vertices_internal <- function (graph) {
     # The next line is the time-killer here, which is why this is cached
     indx <- which (!duplicated (verts$id))
     verts <- verts [indx, , drop = FALSE] # nolint
-    verts$n <- seq (nrow (verts)) - 1
+    verts$n <- seq_len (nrow (verts)) - 1
 
     return (verts)
 }
@@ -323,7 +323,7 @@ dodgr_components <- function (graph) {
         gr_cols <- dodgr_graph_cols (graph)
         graph2 <- convert_graph (graph, gr_cols)
         if (is.na (gr_cols$edge_id)) {
-            graph2$edge_id <- seq (nrow (graph2))
+            graph2$edge_id <- seq_len (nrow (graph2))
         }
         cns <- rcpp_get_component_vector (graph2)
 
@@ -384,7 +384,7 @@ dodgr_sample <- function (graph, nverts = 1000) {
         graph <- graph [sort (indx), ]
     }
 
-    attr (graph, "hash") <- digest::digest (graph [[gr_cols$edge_id]])
+    attr (graph, "hash") <- get_hash (graph, contracted = FALSE, force = TRUE)
 
     return (graph)
 }
@@ -448,7 +448,7 @@ dodgr_insert_vertex <- function (graph, v1, v2, x = NULL, y = NULL) {
             graph [index21 + 1, gr_cols$from] <- randid (charvec, 10)
     }
 
-    attr (graph, "hash") <- digest::digest (graph [[gr_cols$edge_id]])
+    attr (graph, "hash") <- get_hash (graph, contracted = FALSE, force = TRUE)
 
     return (graph)
 }
